import { padStart, startsWith, toLower, trimStart } from 'lodash';

import * as urls from '../constants/genomics-urls';

// NCBI APIs
export var efetch = function efetch(db, id) {
  var retmode = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'xml';
  var params = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : '';
  return urls.NCBI_EFETCH_ENDPOINT + '?retmode=' + retmode + '&db=' + db + '&id=' + id + '&' + params;
};

export var esummary = function esummary(db, id) {
  var retmode = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'json';
  var params = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : '';
  return urls.NCBI_ESUMMARY_ENDPOINT + '?retmode=' + retmode + '&db=' + db + '&id=' + id + '&' + params;
};

export var esearch = function esearch(db, term) {
  var retmode = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'json';
  var params = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : '';
  return urls.NCBI_ESEARCH_ENDPOINT + '?retmode=' + retmode + '&db=' + db + '&term=' + term + '&' + params;
};

// NCBI URLs
export var pubmedSearch = function pubmedSearch(term) {
  return urls.PUBMED_URL + '/?term=' + term;
};

export var pubmedEntry = function pubmedEntry(pmid) {
  return urls.PUBMED_URL + '/' + pmid;
};

export var variationViewer = function variationViewer(chr, start, end) {
  var assembly = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'GCF_000001405.25';
  return urls.NCBI_VARIATION_URL + '/?assm=' + assembly + '&chr=' + chr + '&from=' + start + '&to=' + end;
};

export var clinvarSearch = function clinvarSearch(term) {
  return urls.CLINVAR_URL + '/?term=' + term;
};

export var clinvarEntry = function clinvarEntry(id) {
  return urls.CLINVAR_URL + '/variation/' + id;
};

// Genomic URLs
export var cosmicEntry = function cosmicEntry(id) {
  var trimmedId = trimStart(id, 'COSM');
  return urls.COSMIC_URL + '/mutation/overview?id=' + trimmedId;
};

export var ensemblLocus = function ensemblLocus(chr, start, end) {
  var genome = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'grch37';
  var species = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'Homo_sapiens';

  var url = void 0;

  // TODO: This should be switched to a lookup object sicne genome and species are so related

  if (['hg19', 'grch37'].includes(toLower(genome))) {
    url = '' + urls.ENSEMBL_GRCH37_URL;
  } else if (['hg38', 'grch38'].includes(toLower(genome))) {
    url = '' + urls.ENSEMBL_GRCH38_URL;
  } else {
    throw new Error('genome: ' + genome + ' not supported!');
  }

  return url + '/' + species + '/Location/View?db=core;r=' + chr + ':' + start + '-' + end;
};

export var exacVariant = function exacVariant(chr, pos, ref, alt) {
  return urls.EXAC_URL + '/variant/' + chr + '-' + pos + '-' + ref + '-' + alt;
};

export var gnomadVariant = function gnomadVariant(chr, pos, ref, alt) {
  return urls.GNOMAD_URL + '/variant/' + chr + '-' + pos + '-' + ref + '-' + alt;
};

export var ucscLocus = function ucscLocus(chr, start, end) {
  var genome = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'hg19';
  return urls.UCSC_URL + '/cgi-bin/hgTracks?db=' + genome + '&position=chr' + chr + ':' + start + '-' + end;
};

export var dgvRegionSearch = function dgvRegionSearch(chr, start, end) {
  return urls.DGV_URL + '/gb2/gbrowse/dgv2_hg19/?name=chr' + chr + ':' + start + '-' + end + ';search=Search';
};

// Gene Resources
export var geneCardsEntry = function geneCardsEntry(gene) {
  return urls.GENE_CARDS_URL + '/cgi-bin/carddisp.pl?gene=' + gene;
};

export var ghrEntry = function ghrEntry(gene) {
  return urls.GHR_URL + '/gene/' + gene;
};

export var hgncEntry = function hgncEntry(gene) {
  return urls.HGNC_URL + '/cgi-bin/search?search_type=all&search=' + gene;
};

export var geneReviewsEntry = function geneReviewsEntry(gene) {
  return urls.GENEREVIEWS_URL + '/?term=' + gene;
};

export var gtexEntry = function gtexEntry(gene) {
  return urls.GTEX_URL + '/home/gene/' + gene;
};

export var exacGeneEntry = function exacGeneEntry(gene) {
  return urls.EXAC_URL + '/gene/' + gene;
};

export var gnomadGeneEntry = function gnomadGeneEntry(gene) {
  return urls.GNOMAD_URL + '/gene/' + gene;
};

// Clinical Resources
export var decipherVariant = function decipherVariant(chr, start, end) {
  return urls.DECIPHER_URL + '/search?q=' + chr + ':' + start + '-' + end + '#consented-patients/results';
};

export var decipherRegionSearch = function decipherRegionSearch(chr, start, end) {
  return urls.DECIPHER_URL + '/search?q=chr' + chr + ':' + start + '-' + end + '#consented-patients/results';
};

export var hgmdVariant = function hgmdVariant(cdna) {
  return urls.HGMD_URL + '/hgmd/pro/mut.php?acc=' + cdna;
};

export var hgmdGene = function hgmdGene(gene) {
  return urls.HGMD_URL + '/hgmd/pro/gene.php?gene=' + gene;
};

export var omimEntry = function omimEntry(id) {
  return urls.OMIM_URL + '/entry/' + id;
};

export var omimSearch = function omimSearch(term) {
  return urls.OMIM_URL + '/search/?index=entry&search=' + term;
};

// Phenotype Resources
export var hpoEntry = function hpoEntry(id) {
  if (startsWith(id, 'HP:')) {
    return urls.HPO_URL + '/hpoweb/showterm?id=' + id;
  } else {
    var hpoId = padStart(id, 7, '0');
    return urls.HPO_URL + '/hpoweb/showterm?id=HP:' + hpoId;
  }
};

export var orphanetEntry = function orphanetEntry(id) {
  return urls.ORPHANET_URL + '/consor/cgi-bin/OC_Exp.php?Lng=EN&Expert=' + id;
};

// Internal Resources
export var alamutEntry = function alamutEntry(cdna) {
  return urls.ALAMUT_URL + '/show?request=' + cdna;
};

// ClinGen urls
export var clinGenHGNCGene = function clinGenHGNCGene(hgncId) {
  return urls.CLINGEN_URL + '/genes/HGNC:' + hgncId;
};
export var clinGenCondition = function clinGenCondition(mondoId) {
  return urls.CLINGEN_URL + '/conditions/MONDO_' + mondoId;
};